st7/
;**********************************************************
;DYSKRYMINATOR OKIENKOWY
;Program demonstracyjny uzycie przetwornika A/D
;i wyswietlacza LCD: 2 linie x 20 znakw
;**********************************************************

	#include "st7flite29.inc"

;segmenty pamieci
	BYTES
	segment byte at 80-FF 'ram0'
	segment byte at 100-1FF 'stack'
	segment byte at 200-27F 'ram1'
	segment byte at 1000-10FF 'eeprom'
	segment byte at E000-FFDF 'program'
	segment byte at FFE0-FFFF 'intvect'

	BYTES
;deklaracje zmiennych
	segment 'ram0'
del	DS.B
copya	DS.B
vvolts	DS.W

;deklaracje stalych
	WORDS
	segment 'program'
txton	DC.B	"W OKNIE   ",0
txtoff	DC.B	"POZA OKNEM",0
min	DC.W	300
max	DC.W	800

;-------------------------------------------------------
; poczatek programu glownego, funkcja inicjalizacji
; stosu, portow I/O oraz zmiennych
;-------------------------------------------------------
.init
	ld	A,#$FF		;port A jako wyjsciowy
	ld	PADDR,A		;i stan niski na wszystkich wyjsciach
	ld	PAOR,A
	clr	A
	ld	PADR,A

	ld	PBDDR,A		;port B jako wejsciowy
	ld	PBOR,A
	
	ld	A,#$01		;f_CPU = f_OSC/32 @ 1MHz
	ld	MCCSR,A
	
	ld	A,#$12		;f_TIMER = f_CPU
	ld	ATCSR,A
        
	ld	A,#$DF		;konfiguracja 12-bitowego timera
	ld	ATRL,A		;autoreload, przerwanie co ok.1 ms
	ld	A,#$0F
	ld	ATRH,A
	ret
	

;-------------------------------------------------------
; funkcja obslugi przerwania timera B uzywana przez
; delay do odmierzania czasu
;-------------------------------------------------------
.timerb
	ld	A,ATCSR		;odczyt ATCSR w celu zerowania flagi przerwania
	ld	A,del		;do A wartosc opoznienia
	and	A,#$FF		;sprawdzenie, czy A=0
	jreq	timerb_exit
	dec	A
	ld	del,A
timerb_exit
	iret


;-------------------------------------------------------
; funkcja realizujaca opoznienie "del" x 1ms (maks.255ms)
; wielkosc opoznienia w komorce "del"
;-------------------------------------------------------
.delay
	push	A		;zapamietanie akumlatora na stosie
	rim			;zalaczenie przerwan
d_loop
	ld	A,del		;zmienna "del" zmniejszana przez funkcje
	and	A,#$FF		;obslugi przerwania timer'a
	jreq	d_end		;w tej petli funkcja oczekuje na wartosc del=0
	jp	d_loop
d_end
	sim			;wylaczenie przerwan
	pop	A		;odtworzenie zawartosci akumulatora
	ret
	

;-------------------------------------------------------
; wyslanie danej do wyswietlacza, rejestr docelowy
; okreslany przez inne funkcje-tu tylko przeslanie
; zmienna wejsciowa w akumulatorze!
; RS=PA4,EN=PA7,PA0=DB4,PA1=DB5,PA2=DB6,PA3=DB7
;-------------------------------------------------------
.lcd_write        
	push	Y
	ld	Y,A		;zapamietanie akumulatora
	ld	A,PADR		;odczyt stanu portu A (bo moga byc podlaczone do
				;wyprowadzen portow inne obwody,niz wyswietlacz)
	or	A,#$80		;ENABLE=1
	ld	PADR,A
	or	A,#$0F		;ustawienie bitw PA.0-4 na "1" dla operacji AND
	ld	PADR,A
	push	A		;zapamietanie akumulatora (tzn stanu PADR)
	ld	A,Y		;odtworzenie danej do przeslania
	swap	A		;zamiana polowek bajtu - starsza jako 1-sza
	or	A,#$F0		;ustawienie "nieznaczacej" polowy bajtu na "1"
				;dla operacji AND
	ld	copya,A		;zapamietanie zmiennej do przeslania
	pop	A		;odtworzenie stanu PADR
	and	A,copya		;iloczyn logiczny dla bezkolizyjnego zapisu danych
	ld	PADR,A		;wyprowadzenie polowki bajtu
	and	A,#$7F		;zmiana stanu ENABLE - zapis bajtu do LCD
	ld	PADR,A
	or	A,#$80		;ENABLE=1
	ld	PADR,A
	or	A,#$0F		;ustawienie bitw PA.0-4 na "1" dla operacji AND
	ld	PADR,A
	push	A		;zapamietanie stanu PADR
	ld	A,Y		;odtworzenie danej do przeslania
	or	A,#$F0		;ustawienie "nieznaczacej" polowy bajtu na "1"
				;dla operacji AND
	ld	copya,A		;zapamietanie zmiennej do przeslania
	pop	A		;odtworzenie stanu PADR
	and	A,copya		;iloczyn logiczny dla bezkolizyjnego zapisu danych
	ld	PADR,A		;wyprowadzenie polowki bajtu
	and	A,#$7F		;zmiana stanu ENABLE - zapis bajtu do LCD
	ld	PADR,A
	ld	A,#10		;pauza - LCD akceptuje dane
	call	delay
	pop	Y
	ret


;-------------------------------------------------------
; wyslanie zawartosci akumulatora do rejestru
; konfiguracji wyswietlacza LCD
;-------------------------------------------------------
.lcd_reg_write
	push	A		;zapamietanie zawartosci akumlatora
	ld	A,PADR
	and	A,#$6F		;RS(D4)=0,ENABLE(D7)=0
	ld	PADR,A
	pop	A		;wyslanie zawartosci akumulatora do lcd
	call	lcd_write
	ret

;-------------------------------------------------------
; wyslanie zawartosci akumulatora do rejestru
; danych wyswietlacza LCD
;-------------------------------------------------------
.lcd_data_write
	push	A
	ld	A,PADR
	and	A,#$6F		;RS(D4)=0,ENABLE(D7)=0
	or	A,#$10		;RS(D4)=1
	ld	PADR,A
	pop	A
	call	lcd_write
	ret
	
;-------------------------------------------------------
; inicjalizacja wyswietlacza LCD w trybie interfejsu
; o dlugosci slowa 4 bity
;-------------------------------------------------------
.lcd_init
	ld	A,#100		;pauza okolo 100 ms
	ld	del,A
	call	delay
	ld	A,PADR		;RS(D4)=0,ENABLE(D7)=0,dane=0
	and	A,#$60
	ld	PADR,A
	ld	Y,#3		;3-krotne przeslanie 0x03 do lcd
	or	A,#3
	ld	PADR,A
lcd_init_loop    
	ld	A,PADR		;odczyt stanu rejestru danych portu A
	or	A,#$80		;ENABLE=1
	ld	PADR,A
	nop
	nop
	and	A,#$7F		;ENABLE=0
	ld	PADR,A
	ld	A,#5		;opoznienie 5 milisekund
	ld	del,A
	call	delay
	dec	Y		;czy juz powtorzono 3x?
	jrne	lcd_init_loop
	ld	A,#2		;zapisanie "2" do rejestru kontrolnego
	call	lcd_reg_write
	ld	A,#$28		;kolejne wartosci inicjujace ($28,8,1,6,$C)
	call	lcd_reg_write
	ld	A,#8
	call	lcd_reg_write	
	ld	A,#1
	call	lcd_reg_write	
	ld	A,#6
	call	lcd_reg_write	
	ld	A,#$C
	call	lcd_reg_write	
	ret   

;-------------------------------------------------------
; kasowanie ekranu wyswietlacza LCD
;-------------------------------------------------------
.lcd_cls
	push	A
	ld	A,#1
	call	lcd_reg_write
	pop	A
	ret


;-------------------------------------------------------
; ustawienie kursora na wspolrzednych X,Y               
; X i Y podawane w A w formacie A:Y<<4+X
;-------------------------------------------------------
.gotoaxy
	ld	Y,A		;zapamietanie akumulatora w rejestrze "Y"
	and	A,#$0F		;wyodrebnienie parametru X
	push	A		;zapamietanie parametru X na stosie
	ld	A,Y		;odtworzenie akumulatora, wyodrebnienie par.Y
	sra	A		;arytmetyczne przesuniecie w prawo
	sra	A		;najstarszy bit jest zastepowany przez "0"
	sra	A
	sra	A
	cp	A,#0		;skok wykonywany,jesli A=0 (dla Y=0)
	jreq	gzero
	cp	A,#1 		;jesli A=1
	jreq	gone
	cp	A,#2		;jesli A=2
	jreq	gtwo
	cp	A,#3		;jesli A=3
	jreq	gthree
	ret
;jesli Y=0
gzero
	ld	A,#$80
	ld	copya,A		;obliczenie offsetu adresu dla Y=0
	jp	gotoaxy_exit
gone                            
	ld	A,#$C0
	ld	copya,A		;dla Y=1
	jp	gotoaxy_exit
gtwo                     
	ld	A,#$94
	ld	copya,A		;dla Y=2
	jp	gotoaxy_exit
gthree                  
	ld	A,#$D4
	ld	copya,A		;dla Y=3
gotoaxy_exit
	pop	A
	add	A,copya
	call	lcd_reg_write
	ret


;-------------------------------------------------------
;pomiar napiecia (wartosc zmierzona w komorce VVOLTS
;-------------------------------------------------------
volts
	clr	ADCCSR		;inicjacja rejestru kontrolnego, wylaczenie przetwornika
	clr	ADCDRL		;nastawa rejestru czasu pomiaru i wzmacniacza wejsciowego
	ld	A,#$40		;ustawienie szybkosc przetwarzania (SPEED=1)
	ld	ADCCSR,A
	ld	A,ADCDRH		;kasowanie flagi EOC
	ld	A,#$04
	ld	ADCCSR,A		;zalaczenie kanalu pomiarowego AIN4
	or	A,#$20		;uruchomienie pomiaru
	ld	ADCCSR,A
	btjf	ADCCSR,#7,*	;oczekiwanie na zakonczenie pomiaru
	clr	vvolts		;kompozycja starszego bajtu wyniku
	ld	X,vvolts		;ustawienie dwch  najmlodszych bitw przez przesuniecie z flaga C
	ld	A,ADCDRH
	sll	A
	rlc	X
	sll	A
	rlc	X
	ld	vvolts,X		;zapamietanie starszego bajtu
	ld	{vvolts+1},A	;kompozycja mlodszego bajtu suma z dwoma mlodszymi bitami z ADCDRL
	ld	A,ADCDRL
	and	A,#$03		;maskowanie nieznaczacych bitw
	or	A,{vvolts+1}	;sumowanie D0 i D1 wyniku
	ld	{vvolts+1},A	;zapamietanie wyniku pomiaru
	ret


;*******************************************************
;Testowanie, czy zmienna (data > min) i (data < max)
;argumenty:
; - data (2 bajty,testowana liczba)
; - min (wartosc minimalna,2 bajty)
; - max (wartosc maksymalna, 2 bajty)
;rezultaty:
; - C=0, gdy min < data < max inaczej C=1
;*******************************************************
check_min_max
	push	A		;zapamietanie rejestrw roboczych
	push	X
	ld	X,vvolts 	;pobranie do X bardziej znaczacego bajtu do testu
	ld	A,{vvolts+1}	;pobranie do A mniej znaczacego bajtu
	cp	X,max 		;porwnanie bardziej znaczacego bajtu z max (maksimum)
	jrugt	out_of_range 	;jezeli wiekszy, to wyjscie
	jrne	comp_min 	;jezeli rwny, to porwnanie mniej znaczacego bajtu
	cp	A,{max+1}
	jrugt	out_of_range 	;mniej znaczacy bajt wiekszy - wyjscie
comp_min
	cp	X,min 		;to samo dla wartosci minimalnej min
	jrult	out_of_range
	jrne	in_range
	cp	A,{min+1}
	jrult	out_of_range
in_range
	rcf			;zmienna miesci sie w zakresie  C=0
	jra 	check_exit
out_of_range
	scf 			;zmienna poza zakresem  C=1
check_exit
	pop 	X 		;odtworzenie rejestrw roboczych
	pop 	A
	ret

;-------------------------------------------------------
; program glowny, wyswietlenie tekstu zawartego w ROM               
;-------------------------------------------------------
.main
	rsp
	call	init
	call	lcd_init
	call	lcd_cls		;czyszczenie LCD
main0
	clr	A		;kursor na pozycji 0,0
	call	gotoaxy
	call	volts		;pomiar napiecia
	call	check_min_max	;jesli zmienna miescie sie w zakresie, to napis ON
				;(C=0), jesli nie to napis OFF (C=1)
	clr	X		;zerowanie rejestru indeksowego (nie zmienia stanu C!)
	jrc	main2
main1				;wyswietlenie napisu ZALACZONY
	ld	A,(txton,X)
	and	A,#$FF		;czy to koniec napisu (znak=0)?
	jreq	main3
	call	lcd_data_write	;nie,zapis kodu znaku do LCD
	inc	X		;nastepny znak z napisu
	jra	main1
main2				;wyswietlenie napisu WYLACZONY
	ld	A,(txtoff,X)
	and	A,#$FF		;czy to koniec napisu (znak=0)?
	jreq	main3
	call	lcd_data_write	;nie,zapis kodu znaku do LCD
	inc	X		;nastepny znak z napisu
	jra	main2
main3
	ld	A,#200		;opoznienie 0,2 sekundy
	ld	del,A
	call	delay
	jra	main0		;powtorz


;-------------------------------------------------------
; "pusta" funkcja, zawiera tylko instrukcje powrotu
; z obslugi przerwania
;-------------------------------------------------------
.it_ret	iret


	segment 'intvect'
;-------------------------------------------------------
; wektory przerwan
;-------------------------------------------------------
	DC.W	it_ret
	DC.W	it_ret
	DC.W	it_ret
.sci	DC.W	it_ret
.timb	DC.W	timerb
.tima	DC.W	it_ret
.spi	DC.W	it_ret
	DC.W	it_ret
	DC.W	it_ret
.ext3	DC.W	it_ret
.ext2	DC.W	it_ret
.ext1	DC.W	it_ret
.ext0	DC.W	it_ret
	DC.W	it_ret
.soft	DC.W	it_ret
.rst	DC.W	main



	END
