st7/                                    

;**************** (c) 1997 STMicroelectronics **********************
;
;PROJECT:		Application Note : Software UART using 12-bit ART
;COMPILER: 		ST7 ASSEMBLY CHAIN
;MODULE: 		UART_SW.asm
;CREATION DATE: 	30th June 2003
;AUTHOR: 		Micro Application / STMicroelectronics
;
;-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
;
;   THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY. STMicroelectronics 
;   SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL 
;   DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM USE OF THIS SOFTWARE.
;    
;-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
;
;DESCRIPTION:	Software implementation of UART, using only 12 bit ART & 2 I/O's.
;               This driver software is validated on ST7FLITE0 with internal 8Mhz clock
;               having main features,
;               - RS232 Half Duplex
;               - Async. operation
;               - Flexible data format              
;               For basic functionality please read AN first.
; 		The software consists of main 4 routines:
;		-	TX_data
;		-	RX_data
;		-	EI1_Interrupt
;		-	OPCOMP_int  
;               The file LITE0_Map.INC have hardware register map
;
;-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
;| VERSION 1.0:										|
;|	- First release									|
;| 											|
;+--------------------------------------------------------------------------------------+


	TITLE	"UART_SW.asm" 
				; This title will appear on each
				; page of the listing file.          

	MOTOROLA     		; This directive forces the Motorola 
				; format for the assembly (default).
         
	#INCLUDE "LITE0Map.inc" ; This file includes the different registers
				; mapped into Lite0 device
;******************************************************************************
;       Macro definitions
;******************************************************************************
; bit definition of PWM0CSR register ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 #define        CMPF0   $0      ; CMPF0 PWM0 Compare Flag
 				; 1= Upcounter value matches DCR value.

; bit definition of PWMCR register ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #define        OE0	$0      ; OE0 PWM0 Output enable
 				; 0= Output compare mode

; bit definition of ATCSR register ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

 #define        CMPIE    $0      ; CMPIE Compare Interrupt Enable
 				 ; 1= CMPF interrupt enabled.
 #define        CK0      $3      ; Counter Clock Selection
 				 ; When CK0=0, CK1=1 then fCPU.
 #define        CK1      $4      ; Counter Clock Selection
 
; bit definition of PORTA register used for Tx & Rx~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #define        TxD	$3       ; PortA3 for tx data
 #define	RxD	$7       ; PortA7 for receive data

 
; bit definition of the sci_status register ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;             -----------------------------------------------------------------
; sci_status  |       |       |       |       |       |       |       |       |
;             |       |       |       |  TE   |  BS   |  RE   |  BR   |  SP   |
;  register   |       |       |       |       |       |       |       |       |
;             -----------------------------------------------------------------


 #define        SP      $0      ; Sampling Phase of reception mode.
 #define        BR      $1      ; Byte Received flag.
 #define        RE      $2      ; Reception Enable.
 #define        BS      $3      ; Byte Sent flag.
 #define        TE      $4      ; Transmission Enable.

; definition of masks

 #define    Mask_ICAP1   $01	; Mask of PA7 for the sampling on PB7 pin 
 #define    PA7_Int_conf $08	; =EICR for Falling edge only Interrupt 
;+--------------------------------------------------------------------------------------+
;| CONFIGURATION: Variables and constants				|
;|		  defined and used as local ones					|
;+--------------------------------------------------------------------------------------+

	BYTES				; Following addresses are 8 bit length

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	segment byte at 0-7F  'periph'	; Defines peripherals registers segment.

	segment byte at 80-BF 'ram0'	; Defines Zero Page segment

; Variable definitions ********************************************************

.tx_byte        DC.B    0       ; Byte to transmit.
.rx_byte        DC.B    0       ; Reception byte.
.bit_count      DC.B    0       ; Indicate the bit number. 
.sci_status     DC.B    0       ; 5 bits software register that indicates the 
                                ; communication mode.
                                ; See above for the bits definition.
.posn_bit       DC.B    0       ; Indicate the position of the bit to receive.
.tmp_var        DC.B    0       ; Temporary variable used in program
.data_length	DC.B	0       ; data_length + stop_bit


	WORDS			; Following addresses are 16 bit length

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	segment byte at C0-FF	'stack'	; Defines Stack segment

	segment byte at FA00-FFDF 'rom'	; Defines Rom Segment

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         
	WORDS
     
	segment 'rom'

; Constante definitions *******************************************************
; Bit delay count according to baudrate****************************************

; Baud rate selection for a 8MHz internal clock ~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes of the number of timer cycles to create a delay of 
; one byte for a transmission speed of 19200bd.
;.del_1bl        DC.B    $a0     
;.del_1bh        DC.B    $01
; Baud rate selection for a 8MHz internal clock ~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes of the number of timer cycles to create a delay of 
; one byte for a transmission speed of 9600bd.
;.del_1bl        DC.B    $41     
;.del_1bh        DC.B    $03
; Baud rate selection for a 8MHz internal clock ~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes of the number of timer cycles to create a delay of 
; one byte for a transmission speed of 4800bd.
;.del_1bl        DC.B    $82     
;.del_1bh        DC.B    $06
; Baud rate selection for a 8MHz internal clock ~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes of the number of timer cycles to create a delay of 
; one byte for a transmission speed of 2400bd.
.del_1bl        DC.B    $05	    
.del_1bh        DC.B    $0d                                                    

; Half Bit delay count according to baudrate**********************************

; Delay to sample in the middle of the first byte ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes to sample in the middle of the first byte for 
; a transmission speed of 19200bd.
;.del_sampl      DC.B    $10		
;.del_samph      DC.B    $00		
; Delay to sample in the middle of the first byte ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes to sample in the middle of the first byte for 
; a transmission speed of 9600bd.
;.del_sampl      DC.B    $a0
;.del_samph      DC.B    $01
; Delay to sample in the middle of the first byte ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes to sample in the middle of the first byte for 
; a transmission speed of 4800bd.
;.del_sampl      DC.B    $41
;.del_samph      DC.B    $03
; Delay to sample in the middle of the first byte ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low and high bytes to sample in the middle of the first byte for 
; a transmission speed of 2400bd.
.del_sampl      DC.B     $82	
.del_samph      DC.B     $06	

; Configuration of Data length + no. of STOP bits~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;.stop_bit       DC.B    $01 ; STOP bits to transmit/receive = One STOP bit    
.stop_bit        DC.B    $02 ; STOP bits to transmit/receive = Two STOP bits
;.txrx_data_lnth DC.B	 $08 ; data bits to transmit/receive = 7 data bits + START bit 
.txrx_data_lnth DC.B	 $09 ; data bits to transmit/receive = 8 data bits + START bit



; Program code ****************************************************************

;       ********************************************
;       *                                          *
;       *          MAIN-ROUTINES SECTION           *
;       *                                          *
;       ********************************************


.main	
 	CALL	UART_init	; Init I/O's & variables

	LD      A,#'$'          ; '$' is
        LD      tx_byte,A       ; first byte to send.
        CALL	TX_data		; transmit data in tx_byte 

; Main loop, receive & then transmit received data~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
rx_tx    
	CALL	RX_data         ; Receive data in rx_byte
        LD      A,rx_byte       ; 
        LD      tx_byte,A       ; Transmit byte=Received byte.
	CALL	TX_data         ; Transmit data
        JP      rx_tx           ; loop to Rx_Tx

	
;+--------------------------------------------------------------------------------------+
; Program code ****************************************************************

;       ********************************************
;       *                                          *
;       *       SUB-ROUTINES LIBRARY SECTION       *
;       *                                          *
;       ********************************************

;------------------------------------------------------------------------------
;ROUTINE NAME : Init_UART
;INPUT/OUTPUT : None.
;DESCRIPTION  : Init I/O's & variables used for UART
;COMMENTS     : 
;------------------------------------------------------------------------------

UART_init                 
	
;I/O init 
       
        BSET	PADDR,#TxD      ;PA3=TxD defined as output push-pull
        BSET	PAOR,#TxD
        
	BRES	PAOR,#RxD       ;PA7=RxD defined as input floating
        BRES	PADDR,#RxD                     
 
        BSET	PADR,#TxD       ;TxD High

	LD	A,txrx_data_lnth;
        ADD	A,stop_bit
	LD	data_length,A   ;START+data+STOP bits  
        
     	RET     
     	
;------------------------------------------------------------------------------
;ROUTINE NAME : RX_data 
;INPUT/OUTPUT : rx_byte
;DESCRIPTION  : Receive mode routine
;COMMENTS     : Receive data & store it in rx_byte
;------------------------------------------------------------------------------   	
RX_data 

	LD      A,#$04          ; 
        LD      sci_status,A    ; Receive mode.
        LD      A,#0
        LD      bit_count,A     ; Clear bit number (LSB first)
        LD      rx_byte,A       ; Clear receive byte var
        LD      A,#1
        LD      posn_bit,A      ; Initialize posn_bit 

	LD	A,#PA7_Int_conf 
	LD	EICR,A          ; Configure EICR
	        
        BSET	PAOR,#RxD       ;PA7 defined as pull up interrupt input
        BRES	PADDR,#RxD        
	
        RIM                     ; Enable interrupt

; Loop for receiving byte ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
recv_wait
        BTJF    sci_status,#BR,recv_wait    ; Byte received? 

        SIM                     ; Set interrupt mask.

	RET 

;------------------------------------------------------------------------------
;ROUTINE NAME : TX_data  
;INPUT/OUTPUT : tx_byte.
;DESCRIPTION  : Transmit routine
;COMMENTS     : Transmit data in var. tx_byte
;------------------------------------------------------------------------------   		
TX_data 
        LD      A,#$10
        LD      sci_status,A    ; Enter transmission mode.
        CLR     bit_count       ; Clear bit number (LSB first)
        
; Synchronization delay~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
	LD	A,CNTRL         ; Read counter 
	LD	X,CNTRH
	ADD     A,#$30          ; Add a delay to this value.
        LD      Y,A
        LD      A,X
        ADC     A,#$00    
        LD      DCR0H,A         ; An output compare will occur after the delay.
	CALL	CHK_cnt
        LD      DCR0L,Y         ; 
        
	CALL	ENB_ocmp        ; Set clock & interrupt enable	

        RIM                     ; Remove interrupt mask.
 
; Loop for sending byte ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
trans_wait
	BTJF    sci_status,#BS,trans_wait ; Byte sent?  	
        
        SIM                     ; Set interrupt mask.      
        
	RET     	
	
;------------------------------------------------------------------------------
;ROUTINE NAME : CHK_cnt  
;INPUT/OUTPUT : Y reg.
;DESCRIPTION  : Check counter 
;COMMENTS     : 
;------------------------------------------------------------------------------   				
CHK_cnt	
	CP	Y,#$00
	JRNE	chk_cnt_ret	
	INC	Y
chk_cnt_ret	
	RET

;+--------------------------------------------------------------------------------------+	
;Interrupt routine defination		
;+--------------------------------------------------------------------------------------+
;       ********************************************
;       *                                          *
;       *  INTERRUPT SUB-ROUTINES SECTION          *
;       *                                          *
;       ********************************************

;------------------------------------------------------------------------------
;ROUTINE NAME : OpComp_INT (Interrupt routine)
;INPUT/OUTPUT : 
;DESCRIPTION  : Output compare interrupt 
;COMMENTS     : Used for both transmition & reception mode
;------------------------------------------------------------------------------

.OPCOMP_int

	BTJF    PWM0CSR,#CMPF0,opcomp_int_ret ; Is it an OCMP? Yes continue, else return.
 
        BRES    ATCSR,#CMPIE   ; Disable interrupt
        LD      A,bit_count    ;  	
        JRNE    rxtx_routine             ; Check for first bit  
        LD	A,CNTRL        ; Read counter (reference count )
        LD	X,CNTRH        ;
        LD	DCR0H,X        ; Set DCRXX reg's
        LD	DCR0L,A        ;   
rxtx_routine      
	CALL	OPCOMP_appln   ; Receive & transmit routine       
opcomp_int_ret
	
	IRET

;------------------------------------------------------------------------------
;ROUTINE NAME : OPCOMP_appln
;INPUT/OUTPUT : 
;DESCRIPTION  : Handle receive & transmit section
;COMMENTS     : 
;------------------------------------------------------------------------------
OPCOMP_appln
        LD      A,sci_status    ; SCI mode in A.
        CP      A,#$10          ; Transmission mode?
        JRNE    rx_proc          ; Yes continue, else go to RX_bit.
        
; Transmission process ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LD      A,bit_count
        CP      A,#0            ; Check for START?
        JRNE    chk_stop_bit    ; Yes continue, else go to chk_stop_bit

        BRES	PADR,#3		; START bit           
        JP	next_bit_condn

chk_stop_bit
        LD      A,bit_count	; 
        CP      A,#data_length	; STOP bit sent?
        JRNE    chk_data        ; Yes continue, else go to chk_data
        
; Byte sent ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        BSET    sci_status,#BS  ; Byte sent, so set BS 
        
	LD	A,ATCSR         ;
        AND 	A,#$06 		; STOP CLK, Reset CMPIE  
        LD	ATCSR,A

        JRA     opcomp_appln_ret

chk_data
        CP      A,txrx_data_lnth ; all data bit sent? 
        JRULT   tx_data_bit
; Send stop bit ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        BSET    PADR,#TxD        ; Put PA3 pin in high level for STOP bit.
        JRA     next_bit_condn 
        
; Send next data bit ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
tx_data_bit
        BTJF    tx_byte,#0,tx_bit_0  ; Is the next bit to sent equal to 1?
        BSET    PADR,#TxD        ; Put TxD pin in high level (bit=1) for 1 bit.
        JRA     shift_bit

tx_bit_0
        BRES    PADR,#TxD        ; Put TxD pin in low level (bit=0) for 1 bit.
        
; Create a delay of 1 bit ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
shift_bit   
	SRL     tx_byte          ; Put the next bit to send as bit 0 of tx_byte.
next_bit_condn  
	INC     bit_count        ; Increment bit number.
	
        LD      A,DCR0L          ; Read the last OCMP value.
        LD      X,DCR0H
        ADD     A,del_1bl        ; Add a delay of one bit to this value.

        LD      Y,A
        LD      A,X
        ADC     A,del_1bh
        LD      DCR0H,A          ; An OCMP will occur one bit after 

	CALL	CHK_cnt
        LD      DCR0L,Y          ;      

; End of the output compare management ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
enb_cmpie   
	 TNZ 	 PWM0CSR         ; For clearing the CMPF0 BIT/Interrupt.	
	 BSET    ATCSR,#CMPIE    ; 
opcomp_appln_ret
    	 RET	
; Check for receive section~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
rx_proc      
	CP      A,#$05          ; Reception mode?
        JRNE    enb_cmpie       ; Yes continue, else go to enb_cmpie

; Reception mode ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        LD      A,bit_count         
        CP      A,#data_length	; Byte completely received?
        JRUGE   byte_received   ; No continue, else go to byte_received

; Read & check the new bit ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        
        LD      A,PADR          
        RLC	A
        RLC     A
        AND     A,#Mask_ICAP1   ; Check the first sample of RxD pin.
        LD      tmp_var,A
        LD      A,PADR
        RLC	A
        RLC     A
        AND     A,#Mask_ICAP1   ; Check the second sample of RxD pin.
        ADD     A,tmp_var         ; Add it to the first value.
        LD      tmp_var,A
        LD      A,PADR                  
        
     
        RLC	A
        RLC     A
        AND     A,#Mask_ICAP1   ; Check the third sample of RxD pin.
        ADD     A,tmp_var         ; Add it to the former value.
        PUSH	A
;Check START bit~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
       	LD      A,bit_count         
        CP      A,#0         	; Is it START?
        JRNE    stop_chk     	; No continue, with START received
        POP	A	
        CP      A,#$02          ; 
        JRMI    next_bit_condn
; Optional: User can modify here to handle errors~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        		
        JP	next_bit_condn  

stop_chk
       	LD      A,bit_count         
        CP      A,txrx_data_lnth ; Is it STOP?
	JRULT   databit_chk
        POP	A	
        CP      A,#$02           ; Two or three '1' received?
	JRUGE   rx_end_chk
; Optional: User can modify here to handle errors~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        		
;        JP	byte_received  
rx_end_chk
	LD      A,bit_count
	INC	A         
        CP      A,#data_length
  	JRUGE   byte_received
  	JP      next_bit_condn      		
;Data part~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	        
databit_chk
        POP	A	
        CP      A,#$02          ; Two or three '1' received?
        JRMI    receive_0       ; Yes continue, else go to receive_0
        LD      A,rx_byte
        ADD     A,posn_bit
        LD      rx_byte,A       ; Store received data in rx_byte
        
; Create a delay of 1 bit ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
receive_0
        SLL     posn_bit
        JRA     next_bit_condn  

; New byte received ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
byte_received
        BSET    sci_status,#BR   ; Byte received  

	LD	A,ATCSR
        AND 	A,#$06 		 ; STOP CLK, Reset CMPIE  
        LD	ATCSR,A

        JRA     opcomp_appln_ret

;------------------------------------------------------------------------------
;ROUTINE NAME : IpCap_INT (Interrupt routine)
;INPUT/OUTPUT : 
;DESCRIPTION  : Input capture used to detect START bit 
;COMMENTS     : 
;------------------------------------------------------------------------------
.EI1_int
        LD      A,CNTRL         ; Check the current value of counter
        LD      X,CNTRH
        ADD     A,del_sampl     ; Add a delay to have a OCMP after 0.5 bit 

        LD      Y,A             ; after the input capture
        LD      A,X
        ADC     A,del_samph
        LD	DCR0H,A         ; An output compare will occur in 0.5 bit.
        	                ; Remove pending OCMP interrupt.
 	CALL	CHK_cnt
        LD      DCR0L,Y 

; Enter in sampling mode ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	BSET    sci_status,#SP  ; Enter sampling mode.
	BRES	PAOR,#RxD       ; PA7 defined as input floating,
        BRES	PADDR,#RxD      ; No interrupt
        CALL	ENB_ocmp	; Set counter clock & Enable OCMP interrupt
.Dummy
	IRET 

ENB_ocmp 
	TNZ	 PWM0CSR        ; Clear pending OCMP interrupt
	BRES	 PWMCR,#OE0     ; Enable OPC function  
	LD	 A,#$11		; CMPIE=1:Compare Interrupt Enable.
 	LD	 ATCSR,A        ; Counter Clock = fCPU
	RET	
	
	
;+--------------------------------------------------------------------------------------+
	segment byte at FFE0-FFFF 'vectit' ; Defines Vector Interrupt Segment
;+--------------------------------------------------------------------------------------+
;| INTERRUPT VECTORS                                                                    |
;+--------------------------------------------------------------------------------------+

	DC.W	Dummy			;FFE0-FFE1h location 
	DC.W	Dummy			;FFE2-FFE3h location
	DC.W	Dummy			;FFE4-FFE5h location  
	DC.W	Dummy			;FFE6-FFE7h location  
	DC.W	Dummy			;FFE8-FFE9h location  
	DC.W	OPCOMP_int		;FFEA-FFEBh location  
	DC.W	Dummy			;FFEC-FFEDh location
	DC.W	Dummy			;FFEE-FFEFh location
	DC.W	Dummy			;FFF0-FFF1h location 
	DC.W	Dummy			;FFF2-FFF3h location                       
	DC.W	Dummy			;FFF4-FFF5h location  
	DC.W	EI1_int			;FFF6-FFF7h location             
	DC.W	Dummy			;FFF8-FFF9h location                      
	DC.W	Dummy			;FFFA-FFFBh location                      
	DC.W	Dummy			;FFFC-FFFDh location                       
	DC.W	main			;FFFE-FFFFh location


        END
