st7/
;Program demonstracyjny demonstrujacy uzycie
;ukladow ekspanderow MCP23S08 na plytce ZL1ST7

	MOTOROLA			;format MOTOROLA (.s19)

	#include "st7flite29.inc"

;segmenty pamieci
	BYTES
	segment byte at 80-FF 'ram0'
	segment byte at 100-1FF 'stack'
	segment byte at 200-27F 'ram1'
	segment byte at 1000-10FF 'eeprom'
	segment byte at E000-FFDF 'program'
	segment byte at FFE0-FFFF 'intvect'



;U3
U3_wr_addr	EQU	%01000000
U3_rd_addr	EQU	%01000001
;U4 (led)
U4_wr_addr	EQU	%01000010
U4_rd_addr	EQU	%01000011

;MCP registers
MCP_IODIR	EQU	$00
MCP_IPOL		EQU	$01
MCP_GPINTEN	EQU	$02
MCP_DEFVAL	EQU	$03
MCP_INTCON	EQU	$04
MCP_IOCON	EQU	$05
MCP_GPPU		EQU	$06
MCP_INTF		EQU	$07
MCP_INTCAP	EQU	$08
MCP_GPIO		EQU	$09
MCP_OLAT		EQU	$0A

;definicje stalych
low		EQU	0
high		EQU	1


	BYTES
;deklaracje zmiennych
	segment 'ram0'
buf		DS.B
del		DS.B
vvolts		DS.W
sub_a		DS.W
sub_b		DS.W
roznicaw		DS.W
decims		DS.B	6

;deklaracje stalych
	WORDS
	segment 'program'

;tablica stanw dioc led
leds		DC.B	%00000000,%00000001,%00000011,%00000111,%00001111,%00011111,%00111111,%01111111

;-------------------------------------------------------
; inicjalizacja trybw pracy portw
; PA5 - wyjscie, reszta wejscia
; PB0,PB3 - wyjscia, reszta wejscia
;-------------------------------------------------------
PADR_init	EQU	%00100000
PADDR_init	EQU	%00100000
PAOR_init	EQU	%00100000
PBDR_init	EQU	%00000000
PBDDR_init	EQU	%00001010
PBOR_init	EQU	%00001010

PORTS_INIT MACRO
	ld	A,#PBDDR_init		;inicjalizacja portu B
	ld	PBDDR,A
	ld	A,#PBOR_init
	ld	PBOR,A
	ld	A,#PBDR_init
	ld	PBDR,A

	ld	A,#PADDR_init		;inicjalizacja portu A
	ld	PADDR,A
	ld	A,#PAOR_init
	ld	PAOR,A
	ld	A,#PADR_init
	ld	PADR,A
	MEND


;-------------------------------------------------------
; inicjacja SPI:
;   CLK = fcpu/3, divider = wylaczony, cpol = 0,
;   cpha = 0, przerwania = wylaczone, tryb = master
SPISR_init	EQU	%00000011
SPICR_init	EQU	%00010000
SPI_enable	EQU	%01000000
;-------------------------------------------------------
SPI_INIT	MACRO
	ld	A,#SPISR_init
	ld	SPISR,A
	ld	A,#SPICR_init
	ld	SPICR,A
	or	A,#SPI_enable
	ld	SPICR,A
	MEND


;-------------------------------------------------------
; makro ustawiajace sygnal CS dla MCP23S08
;-------------------------------------------------------
CS_HIGH	MACRO
	ld	A,PADR
	or	A,#%00100000	;PA5=1
	ld	PADR,A
	MEND
	
CS_LOW	MACRO
	ld	A,PADR		;odczyt stanu rejestru danych
	and	A,#%11011111
	ld	PADR,A
	MEND


;-------------------------------------------------------
; wyslanie bajtu danych przez SPI
; bajt do przeslania w akumulatorze
;-------------------------------------------------------
spi_send
	ld	SPIDR,A
	btjf	SPISR,#7,*	;oczekiwanie na wyslanie bajtu
	ld	A,SPIDR		;dla wyzerowania SPIF
	ret


;-------------------------------------------------------
; inicjacja ukladow MCP23S08 (oba jako wyjsciowe,
; bez przerwan)
; nastawy: JP5 2-3, JP6 2-3, JP7 1-2, JP12 1-2, JP14 2-3
;-------------------------------------------------------
MCP_set
	callr	spi_send
	clr	A		;programowanie rejestru kierunku (jego adres to 0)
	callr	spi_send		;wyslanie adresu rejestru
	callr	spi_send		;IODIR=0
	callr	spi_send		;IPOL=0
	callr	spi_send		;GPIINTEN=0
	callr	spi_send		;DEFVAL=0
	callr	spi_send		;INTCON=0
	ld	A,#%00001000	;IOCON: auto-inkrementacja,adres zalaczony,
				;sygnalizacja int: stan niski
	callr	spi_send
	clr	A		;rezystory zasilajace wylaczone
	callr	spi_send
	ret

MCP_INIT	MACRO
	CS_LOW			;wyzerowanie sygnalu CS
	ld	A,#U3_wr_addr	;zaadresowanie ukladu U3 (gpio)
	call	MCP_set		;wyslanie nastaw
	CS_HIGH			;koniec nastaw U3 (CS - stan wysoki)

	CS_LOW			;teraz sekwencja nastaw U4 (ten sam tryb, co U3)
	ld	A,#U4_wr_addr	;zaadresowanie ukladu U4 (gpio)
	call	MCP_set		;wyslanie nastaw
	CS_HIGH			;koniec nastaw U4
	MEND


;-------------------------------------------------------
; inicjacja timera Lite (timer B)
;-------------------------------------------------------
TIMB_INIT MACRO
	ld	A,#$01		;f_CPU = f_OSC/32 @ 1MHz
	ld	MCCSR,A
	ld	A,#$12		;f_TIMER = f_CPU
	ld	ATCSR,A
        	ld	A,#$DF		;konfiguracja 12-bitowego timera
	ld	ATRL,A		;autoreload, przerwanie co ok.1 ms
	ld	A,#$0F
	ld	ATRH,A
	MEND


;-------------------------------------------------------
; funkcja obslugi przerwania timera B uzywana przez
; delay do odmierzania czasu
;-------------------------------------------------------
timerb
	ld	A,ATCSR		;odczyt ATCSR w celu zerowania flagi przerwania
	ld	A,del		;do A wartosc opoznienia
	and	A,#$FF		;sprawdzenie, czy A=0
	jreq	timerb_exit
	dec	A
	ld	del,A
timerb_exit
	iret


;-------------------------------------------------------
; funkcja realizujaca opoznienie "del" x 1ms (maks.255ms)
; wielkosc opoznienia w komorce "del"
;-------------------------------------------------------
delay
	push	A		;zapamietanie akumlatora na stosie
	rim			;zalaczenie przerwan
d_loop
	ld	A,del		;zmienna "del" zmniejszana przez funkcje
	and	A,#$FF		;obslugi przerwania timer'a
	jreq	d_end		;w tej petli funkcja oczekuje na wartosc del=0
	jp	d_loop
d_end
	sim			;wylaczenie przerwan
	pop	A		;odtworzenie zawartosci akumulatora
	ret
	

;-------------------------------------------------------
;pomiar napiecia (wartosc zmierzona w komorce VVOLTS
;-------------------------------------------------------
volts
	clr	ADCCSR		;inicjacja rejestru kontrolnego, wylaczenie przetwornika
	clr	ADCDRL		;nastawa rejestru czasu pomiaru i wzmacniacza wejsciowego
	ld	A,#$40		;ustawienie szybkosc przetwarzania (SPEED=1)
	ld	ADCCSR,A
	ld	A,ADCDRH		;kasowanie flagi EOC
	ld	A,#$06
	ld	ADCCSR,A		;zalaczenie kanalu pomiarowego AIN6
	or	A,#$20		;uruchomienie pomiaru
	ld	ADCCSR,A
	btjf	ADCCSR,#7,*	;oczekiwanie na zakonczenie pomiaru
	clr	vvolts		;kompozycja starszego bajtu wyniku
	ld	X,vvolts		;ustawienie dwch  najmlodszych bitw przez przesuniecie z flaga C
	ld	A,ADCDRH
	sll	A
	rlc	X
	sll	A
	rlc	X
	ld	vvolts,X		;zapamietanie starszego bajtu
	ld	{vvolts+1},A	;kompozycja mlodszego bajtu suma z dwoma mlodszymi bitami z ADCDRL
	ld	A,ADCDRL
	and	A,#$03		;maskowanie nieznaczacych bitw
	or	A,{vvolts+1}	;sumowanie D0 i D1 wyniku
	ld	{vvolts+1},A	;zapamietanie wyniku pomiaru
	ret


;-------------------------------------------------------
; konwersja wyniku pomiaru z komrek vvolts na postac
; dla skali diodowej LED (8 diod)
; wynik pomiaru 0..1023 tj. 1024 kroki, rozdzielczosc
; skali LED to 8 diod, wiec 1024/8 = 1 LED to 7 bitow
; dzielenie przez 128 to przez 7
;-------------------------------------------------------
convert
	srl	vvolts
	rrc	{vvolts+1}
	srl	vvolts
	rrc	{vvolts+1}
conv_1
	ld	A,#5
	rrc	{vvolts+1}
	dec	A
	jrne	conv_1
	ld	X,{vvolts+1}
	ld	A,(leds,X)
	ret


;-------------------------------------------------------
; poczatek programu glownego, funkcja inicjalizacji
; stosu, portow I/O oraz zmiennych
;-------------------------------------------------------
.main
	rsp
	TIMB_INIT		;inicjalizacja timera B
	PORTS_INIT		;inicjalizacja PA i PB
	SPI_INIT			;inicjalizacja portu SPI
	MCP_INIT			;inicjalizacja ukladu MCP23S08
	
	CS_LOW			;wylaczenie autoinkrementacji adresu
	ld	A,#U4_wr_addr
	call	spi_send
	ld	A,#MCP_GPIO
	call	spi_send
	ld	A,#%00101000
	call	spi_send

	CS_HIGH
	nop			;krtka pauza na akceptacje nastawy
	nop
	CS_LOW			;wybr rejestru GPIO w ukladzie U4
	ld	A,#U4_wr_addr
	call	spi_send
	ld	A,#MCP_GPIO
	call	spi_send

mloop
	call	volts		;pomiar napiecia
	call	convert		;konwersja na skale liniowa
	call	spi_send		;zapis do GPIO ukladu U4

	jra	mloop


;-------------------------------------------------------
; "pusta" funkcja, zawiera tylko instrukcje powrotu
; z obslugi przerwania
;-------------------------------------------------------
it_ret	iret


	segment 'intvect'
;-------------------------------------------------------
; wektory przerwan
;-------------------------------------------------------
	DC.W	it_ret
	DC.W	it_ret
	DC.W	it_ret
.sci	DC.W	it_ret
.timb	DC.W	timerb
.tima	DC.W	it_ret
.spi	DC.W	it_ret
	DC.W	it_ret
	DC.W	it_ret
.ext3	DC.W	it_ret
.ext2	DC.W	it_ret
.ext1	DC.W	it_ret
.ext0	DC.W	it_ret
	DC.W	it_ret
.soft	DC.W	it_ret
.rst	DC.W	main


	END
